import re
import urllib
from django.template import Library
from django.template.loader import get_template
from django_simple_tags.utils import get_related_model_field

try:
    from django_middleware_global_request.middleware import get_request
except ImportError:
    def get_request():
        return None


register = Library()


@register.simple_tag
def django_horizontal_list_filter(cl, spec):
    request = get_request()
    tpl = get_template(spec.template)
    return tpl.render({
        "cl": cl,
        "title": spec.title,
        "choices": list(spec.choices(cl)),
        "spec": spec,
        "reqeust": request,
    })


@register.simple_tag
def django_horizontal_list_filter_search_box_label(cl):
    model = cl.model
    model_admin = cl.model_admin
    search_fields = model_admin.search_fields
    search_labels = []
    for search_field in search_fields:
        related_model, field = get_related_model_field(model, search_field)
        label = str(field.verbose_name)
        search_labels.append(label)
    return "/".join(search_labels)


@register.simple_tag
def django_horizontal_list_filter_turn_mptt_padding_style_to_padding_string(cl, spec, choice):
    padding_style = choice.get("padding_style", None)
    if not padding_style:
        return ""
    padding = int(re.findall("\d+", padding_style)[0])
    model_admin = cl.model_admin
    mptt_level_indent = getattr(model_admin, "mptt_level_indent", spec.mptt_level_indent)
    if not mptt_level_indent:
        return ""
    return "----" * (padding // mptt_level_indent)

@register.simple_tag
def django_horizontal_list_filter_clean_query_string(cl, spec, query_string):
    if not query_string:
        return ""
    if query_string.startswith("?"):
        query_string = query_string[1:]
    qsl = urllib.parse.parse_qsl(query_string)
    expected_parameters = spec.expected_parameters()
    qsl_new = []
    for k,v in qsl:
        if k in expected_parameters:
            qsl_new.append((k, v))
    query_string = urllib.parse.urlencode(qsl_new)
    return "?" + query_string



from django.apps import AppConfig


class DjangoHorizontalListFilterConfig(AppConfig):
    name = 'django_horizontal_list_filter'

    def ready(self):
        # inject our css&js into media
        from django.contrib.admin import ModelAdmin
        from django import forms
        from django.utils.translation import get_language
        from django.contrib.admin.widgets import SELECT2_TRANSLATIONS
        from django.conf import settings

        extra_js = [
            "admin/js/vendor/jquery/jquery.js",
            "django-horizontal-list-filter/js/django-horizontal-list-filter.js",
        ]
        extra_css = []

        DJANGO_HORIZONTAL_LIST_FILTER_USE_SELECT2 = getattr(settings, "DJANGO_HORIZONTAL_LIST_FILTER_USE_SELECT2", False)
        if DJANGO_HORIZONTAL_LIST_FILTER_USE_SELECT2:
            extra_js.append("admin/js/vendor/select2/select2.full.js")
            extra_js.append("django-horizontal-list-filter/js/django-horizontal-list-filter-use-select2.js")
            extra_css.append("admin/css/vendor/select2/select2.css")
            SELECT2_LANGUAGE_CODE = SELECT2_TRANSLATIONS.get(get_language())
            if SELECT2_LANGUAGE_CODE:
                extra_js.append("admin/js/vendor/select2/i18n/%s.js" % SELECT2_LANGUAGE_CODE)
        
        extra_js += [
            "admin/js/jquery.init.js",
        ]
        extra_css += [
            "django-horizontal-list-filter/css/django-horizontal-list-filter.css",
        ]
        
        ModelAdmin.__django_horizontal_list_filter_old_media = ModelAdmin.media
        ModelAdmin.__django_horizontal_list_filter_old_get_list_filter = ModelAdmin.get_list_filter

        @property
        def media(self):
            return self.__django_horizontal_list_filter_old_media + forms.Media(css={"all": extra_css}, js=extra_js)
        
        def get_list_filter(self, request):
            getattr(request, "is_using_django_horizontal_list_filter", True)
            return ModelAdmin.__django_horizontal_list_filter_old_get_list_filter(self, request)

        ModelAdmin.media = media
        ModelAdmin.get_list_filter = get_list_filter
